#! /usr/bin/python3
import sys
import numpy
import scipy.interpolate
import copy
from plot_me import plot_me

from lmfit import minimize, Parameters
import lmfit




"""
written by Jiri Mares

cite:
    
"""



class MesoPorousD:

    def __init__(self, inputsFN, numGauss = 3, lowHighDiv = 400.0, guessFN = None, kConst10 = 30.0, t0 = 2000.0,
            numOfDistribPoints = 5000 , maxTfromExp = False, plotProg = 0):
        """
        t02s is ignored here
        """
        
        print("###\nDirect fitting of Pore-size distribution, modelled by gaussians\n###")

        self.numOfDistribPoints = numOfDistribPoints

        #self.k = 413.2    #JM
        self.k  = kConst10 * 10.0
        self.T0  = t0
        self.T00 = t0 #not needed here, but for compatibility...

        self.lowHighDiv = lowHighDiv
        inputs = numpy.loadtxt(inputsFN)

        if  guessFN is not None:
            self.guessSetting = numpy.genfromtxt(guessFN, names = True)
        else:
            self.guessSetting = None


        self.salineCurve = scipy.interpolate.interp1d(inputs[:,0], inputs[:,2], kind='cubic',fill_value = 1.0, bounds_error=False)
        self.materialCurve = scipy.interpolate.interp1d(inputs[:,0], inputs[:,1], kind='cubic',fill_value = 1.0, bounds_error=False)
        self.measuredCurveInMaterial = inputs[:,0:2] #t, signal
        self.fitCounter = 0
        self.numGauss = numGauss
        self.poresSizes = numpy.logspace(1, 35, numOfDistribPoints, base=1.3) #up to 9727.8
        #self.poresSizes = numpy.logspace(1, 40, 10000,base=1.3)              #up to 36118  

        self.xDistrib = self.poresSizes
        self.plotProg = plotProg

        if maxTfromExp:
            self.T00 = numpy.max(inputs[:,0])
        else:
            self.T00 = self.T0 + 0.0
        self.minT  = numpy.min(inputs[:,0])
        self.tempPoints          = numpy.linspace(self.minT , self.T0, numOfDistribPoints)  # here only for the conventional


        self.do_conventional()
                                                                              # Temperature derivative
        #sys.exit()

    def do_conventional(self):

        meltingCurveDerivativeInterpolator =  self.materialCurve._spline.derivative(nu=1)
        xy1 = numpy.zeros((self.numOfDistribPoints,2),float)
        someTempPoints          = numpy.linspace(self.minT, self.T00, self.numOfDistribPoints) 
        xy1[:,1] = meltingCurveDerivativeInterpolator( someTempPoints)[:,0]
        xy1[:,0] = someTempPoints[:]

        plot_me(xyScatter =  None  , xyLine = xy1 , title = "PMD as a (spline-interpolated) derivative of a melting curve"  , fileName = "meltingCurvePoreDerivative" , xlim=[self.minT, self.T00], xl = "T[K]", yl="derivative"  )
        self.melting_point_dist_to_pore_size_distr(mpd = xy1[:,1], namePrefac = "conventional_")


    def melting_point_dist_to_pore_size_distr(self, mpd, namePrefac = ""):
        """
        mpd = psd / k * r**2 # r corresponds to the depressed melting point 
        psd = mpd * k / r**2 # eq. 11
        """

        k= self.k

        DeltaT = self.T00 - self.tempPoints    #tempPoints go to T00 (here max(meas points))
        DeltaT0 = self.T0 - self.tempPoints    #This goes to minus shift... e.g. to 2.85
        #remove the (near)-zero
        DeltaT = numpy.where(numpy.abs(DeltaT)  < 1.0e-6, 1.0e-3, DeltaT) 
        DeltaT0 = numpy.where(numpy.abs(DeltaT0)< 1.0e-6, 1.0e-3, DeltaT0) 
        d  = 2.0 * self.k/(DeltaT)             # this goes to positive (near to) infinite values
        d0 = 2.0 * self.k/(DeltaT0)            # if there was positive shift, then, d0 goes to minus values!

        # converting to pore-size distribution for cylindrical cavity model
        
        eq11 = True

        if eq11:
            psd  = mpd  * k/(d**2 /4.0)   # mpd is here the Melting point distribution
            psd0 = mpd  * k/(d0**2/4.0)   # this should be the result comparable with the publication, if suitable shift there

        else: 
            sys.exit()
            #psd  = 2.0* k/(DeltaT)  * mpd # mpd is here the Melting point distribution
            #psd0 = 2.0* k/(DeltaT0) * mpd # this should be the result comparable with the publication, if suitable shift there


        psdM = numpy.zeros((len(psd),2),float)
        psdM[:,1] = psd[:]
        psdM[:,0] = d[:]
        # not printing
        #plot_me(xyScatter =  None  , xyLine = psdM , title = "pore size distribution, no shift applied"  , fileName = "poresizedistribution" , xlim = [0.0, self.lowHighDiv] , xl = r"d [$\AA$]", yl="normalized distribution"  )
        #numpy.savetxt("poreSizeDistributionCSV.txt", psdM)

        psdM0 = numpy.zeros((len(psd0),2),float)
        psdM0[:,1] = psd0[:]
        psdM0[:,0] = d0[:]

        highIndex = int(0.95 * len(psd0))
        maxOfY = numpy.max(psd0[:highIndex])

        shiftTXT =  "%6.3f K" % self.T00
        if abs(self.T00-self.T0) < 1.0e-5:
            titleHere = namePrefac + " pore size distribution"
        else:
            titleHere = namePrefac + " pore size distribution, (if sidi), with wanted reference of "+ shiftTXT

        plot_me(xyScatter =  None  , xyLine = psdM0 , title = titleHere  , fileName = namePrefac + "poresizedistribution0", xlim = [0.0, self.lowHighDiv] , xl = r"d [$\AA$]", yl="normalized distribution" , ylim = [0.0, maxOfY ] )
        numpy.savetxt(namePrefac + "poreSizeDistributionCSV0.txt", psdM0)

        return psdM0


    
    def gaussian_my(self, x, center = 0.0, sigma = 1.0, amplitude = 10.0):
        """
        https://en.wikipedia.org/wiki/Normal_distribution
        """
        g = amplitude/(sigma * numpy.sqrt(2.0*numpy.pi)) *   numpy.exp(-0.5 * ((x-center)/sigma )**2  )
        
        return  g
    
    def pore_size_distrib_from_gaussians(self, params):
    

        numGauss   = self.numGauss 


        distribDiscrete = numpy.zeros( len(self.xDistrib),float)


        integral = 0.0
        for i in range(0,numGauss):
            cn = "center"    + str(i)
            sn = "sigma"     + str(i)
            an = "amplitude" + str(i)
            cnn = params[cn].value
            snn = params[sn].value
            ann = params[an].value
            oneGaussian = self.gaussian_my(x = self.xDistrib, center = cnn, sigma = snn, amplitude = ann)
    
            distribDiscrete += oneGaussian
            integral += ann

    
        distribDiscrete /= integral
    
        return distribDiscrete
    
    
    def Sb_at_pore(self, T, d ):
        """
        deltaT = k/r
        deltaT = 2*k/d
        r in Angstrom
        """
        k = self.k
        salineCurve = self.salineCurve

        deltaT = 2.0*k/d
        

        shiftedTemperatureOfCurve        =      T  + deltaT
        s = salineCurve(x = shiftedTemperatureOfCurve)
    
        return s
    
    def nmrSignalCPMGFiltered(self, params, T ):
        """
        S, of eq 9
        """
        poreSizes = self.poresSizes

    
        psd = self.pore_size_distrib_from_gaussians(params)
        dSAtT = psd  * self.Sb_at_pore(T, d = poreSizes )
        S  = numpy.trapz(dSAtT, x = poreSizes)

        return S
    
    def give_model_curve(self, params):

        measuredCurve = self.measuredCurveInMaterial
        salineCurveInterpolator = self.salineCurve
        numGauss = self.numGauss

    
        modelCurve = numpy.zeros(len(measuredCurve), float)

        for i  in range(0, len(measuredCurve)):
            T               = measuredCurve[i,0]
            modelSignal = self.nmrSignalCPMGFiltered(params, T)
            modelCurve[i] =  modelSignal

        modelCurve /= numpy.max(modelCurve)
        return modelCurve

    def residual(self, params):
        """no uncertranity given
            otherwise woudl return error/uncertranity
        """

        salineCurveInterpolator = self.salineCurve
        measuredCurve = self.measuredCurveInMaterial
        poreSizesSampled = self.poresSizes

        modelCurve = self.give_model_curve(params)
        error = modelCurve - measuredCurve[:,1]
    
        if self.plotProg > 0:
            if (self.fitCounter % self.plotProg) == 0:
                err = numpy.linalg.norm(error)
                errtxt = "%9.7f" % err
                print("plotting...")
                itxt = "%8.8d" % self.fitCounter
                xy = numpy.zeros((len(measuredCurve),2),float)
                xy[:,0] = measuredCurve[:,0]
                xy[:,1] = modelCurve[:]
                plot_me(xyScatter =  measuredCurve  , xyLine = xy     ,title = "Melting curve: Model vs Measured. Error: " + errtxt, fileName = itxt+"meltingCurveModelVsMeasured",  xl = "T [K]", yl="normalized, CPMG-filtered signal" )
           
        if (self.fitCounter  == 0):
            err = numpy.linalg.norm(error)
            errtxt = "%9.7f" % err
            print("plotting...")
            itxt = "%8.8d" % self.fitCounter
            saline = numpy.zeros((len(measuredCurve),2),float)
            saline[:,1] = self.salineCurve(measuredCurve[:,0])
            saline[:,0] = measuredCurve[:,0]
            plot_me(xyScatter = saline, xyLine = measuredCurve ,title = "Melting curve, bulk saline _in_pores_ as inputs", fileName = itxt+"meltingCurveModelVsSaline",  xl = "T [K]", yl="normalized, CPMG-filtered signal" )
            print("params: ", itxt, errtxt)
            params.pretty_print()
            #reporter.reporter(itxt, " error: ",  errtxt , "\n" ,params.pretty_repr(), "\n\n")
           
        self.fitCounter += 1
    
        return error
    
    
    def prepare_starting_params(self):
        """
        """

        params = Parameters()
        numGauss = self.numGauss
        centers    =  numpy.linspace(1,200, numGauss)
        sigmas     =  numpy.linspace(1,100, numGauss)

        largestPoreProbed = numpy.max(self.poresSizes)
        if self.guessSetting is not None:
            if len(self.guessSetting) >= numGauss: # if larger, ignore the number of gaussians

                lowDone  = False
                highDone = False
                for i in range(0, len(self.guessSetting)):
                    cn = "center"    + str(i)
                    sn = "sigma"     + str(i)
                    an = "amplitude" + str(i)
                    cnn = self.guessSetting["c"][i]
                    snn = self.guessSetting["s"][i]
                    ann = self.guessSetting["a"][i]
                    if cnn < 50.0:
                        if not lowDone:
                            params.add(cn, value= cnn   , min=0.1, max=50.0)
                            params.add(sn, value= snn   , min=0.1, max=150.0)
                            params.add(an, value= ann    ,min=0.001, max=1.0)
                            lowDone = True
                        else:
                            params.add(cn, value= cnn   , min=0.1, max= self.lowHighDiv )
                            params.add(sn, value= snn   , min=0.1, max= 450.0)
                            params.add(an, value= ann    ,min=0.001, max=1.0)
                    elif cnn <= self.lowHighDiv:
                        params.add(cn, value= cnn   , min=0.1, max= self.lowHighDiv )
                        params.add(sn, value= snn   , min=0.1, max= 450.0)
                        params.add(an, value= ann    ,min=0.001, max=1.0)
                    elif cnn >= self.lowHighDiv:
                        if not highDone:
                            params.add(cn, value= cnn   , min=self.lowHighDiv, max= largestPoreProbed)
                            params.add(sn, value= snn   , min=1.0, max= 3000.0)
                            params.add(an, value= ann    ,min=0.001, max=1.0)
                            highDone = True
                        else:
                            params.add(cn, value= cnn   , min=0.1 * cnn, max= min(largestPoreProbed, 10.0*cnn))
                            params.add(sn, value= snn   , min=0.1 *snn , max= 10. * snn)
                            params.add(an, value= ann   , min=0.1 *ann , max= min(10. * ann, 1.0))
                           
            else:
                print("less guess params given than wanted gaussians")
                sys.exit()

        else:
            # the low gaussian
            if numGauss > 2:
                i = 0
                cn = "center"    + str(i)
                sn = "sigma"     + str(i)
                an = "amplitude" + str(i)
                params.add(cn, value= 25.0   , min=0.1, max=50.0)
                params.add(sn, value= 10.0   , min=0.1, max=150.0)
                params.add(an, value=0.5     ,min=0.001, max=1.0)
            
            # the high (bulk) gaussian
            if numGauss > 2:
                i = 1
            else:
                i = 0
            self.lowHighDiv
            cn = "center"    + str(i)
            sn = "sigma"     + str(i)
            an = "amplitude" + str(i)
            params.add(cn, value= 5000   , min=self.lowHighDiv, max= largestPoreProbed)
            params.add(sn, value= 5000   , min=1.0, max=3000.0)
            params.add(an, value=0.5          ,min=0.001, max=1.0)
            
            #other gaussians
            if numGauss > 2:
                contWith = 2
            else:
                contWith = 1

            for i in range(contWith, numGauss):
                cn = "center"    + str(i)
                sn = "sigma"     + str(i)
                an = "amplitude" + str(i)
                params.add(cn, value= 1.5* self.lowHighDiv/float(i)   , min=.1, max=self.lowHighDiv)
                params.add(sn, value= 10   , min=1.0, max=450.0)
                params.add(an, value=0.5          ,min=0.001, max=1.0)
        self.params = params
        print("starting params:") 
        params.pretty_print()
        return None
    
    def fit_pore_distr(self, algorithm ):
    
        numGauss= self.numGauss
        params = self.params

        mini = lmfit.Minimizer(self.residual, params)
        out1 = mini.minimize(method = algorithm)
        
        withResults = out1.params 
        print("Results:")
        withResults.pretty_print()
        print()

        normOfErr = numpy.linalg.norm(out1.residual) #was 0.7806
        print("norm of Err: ", normOfErr)

        #plot the resulting pore distribution
        curve = self.pore_size_distrib_from_gaussians(out1.params )
        resultC = numpy.zeros((len(self.poresSizes),2),float)
        resultC[:,0] = self.poresSizes[:]
        resultC[:,1] = curve[:]

        plot_me(xyScatter =  resultC  , xyLine = resultC  ,title = "Pore size distribution"  , fileName =
                "result_pore_distrib", xlim = [0.0, self.lowHighDiv], xl = r"pore diameter [$\AA$]", yl="normalized distribution")
        numpy.savetxt("resultingDistributionCSV.txt", resultC)
    
        #now the resulting melting curve
        measuredCurve = self.measuredCurveInMaterial
        modelCurve0 = self.give_model_curve(withResults)

        xy0 = numpy.zeros((len(modelCurve0),2),float)
        xy0[:,0] = measuredCurve[:,0]
        xy0[:,1] = modelCurve0[:]
    
        errtxt = "%9.7f" % normOfErr
        plot_me(xyScatter = self.measuredCurveInMaterial, xyLine = xy0 ,title = "Melting curve: Model vs Measured. Err: " + errtxt , fileName = "meltingCurveModelVsMeasuredFinal", xl = "T [K]", yl="normalized, CPMG-filtered signal" )
    
        #now the resulting melting point distribution
        k = self.k
        deltaTRange = k/self.poresSizes # just for convenience, preparing T range for the sampled pore sizes
        r = self.poresSizes
        T0 = self.T0
        Tabs = T0 - deltaTRange  #can be also plotted as relative deltaT, but here from 0.0, which may not be always
                                     #wanted? do also another plot ...
    
        meltingPointDistribution = curve/ k * r**2 # r corresponds to the depressed melting point 
        MPDistrib = numpy.zeros((len(meltingPointDistribution),2),float)
        MPDistrib[:,1] = meltingPointDistribution[:]
        MPDistrib[:,0] = Tabs[:]

        highIndex = int(0.95 * len(MPDistrib[:,0]))
        maxOfY = numpy.max(meltingPointDistribution[:highIndex])
        maxOfY = min(1.0, maxOfY)

        plot_me(xyScatter =  None  , xyLine = MPDistrib , title = "melting point distribution"  , fileName =
            "meltingPointDistributionAbsTemp" , xlim=[self.T0-40.0 ,self.T0 + 5], xl = "T [K]", yl="normalized distribution" , ylim = [0.0, maxOfY ] )
        numpy.savetxt("MPDistribAbs", MPDistrib)

        MPDistribRelative = MPDistrib.copy()
        MPDistribRelative[:,0] = -deltaTRange[:]
        plot_me(xyScatter =  None  , xyLine = MPDistribRelative , title = "melting point distribution"  , fileName = "meltingPointDistributionRelTemp" , xlim=[-40,5], xl = r"$-\Delta \,$T [K]", yl="distribution",  ylim = [0.0, maxOfY ] )
        numpy.savetxt("MPDistribRel", MPDistribRelative)


##########################################################################################################################################################
#
#
##########################################################################################################################################################


class MesoPorous(MesoPorousD):


    def __init__(self, inputsFN, numGauss = 3, lowHighDiv = 400.0, guessFN = None, numOfDistribPoints = 80, kConst10 =
            30.0, t0 = 273.0, maxTfromExp = False,  plotProg = 0 ):

        print("###\nFitting of Pore-size distribution through melting-point distribution,  modelled by gaussians\n###")

        self.numOfDistribPoints = numOfDistribPoints
        self.k = kConst10 * 10.0 
        self.T0 = t0

        self.lowHighDiv = lowHighDiv
        inputs = numpy.loadtxt(inputsFN)
        self.minT  = numpy.min(inputs[:,0])

        if maxTfromExp:
            self.T00 = numpy.max(inputs[:,0])
        else:
            self.T00 = self.T0 + 0.0


        if  guessFN is not None:
            self.guessSetting = numpy.genfromtxt(guessFN, names = True)
        else:
            self.guessSetting = None


        self.salineCurve = scipy.interpolate.interp1d(inputs[:,0], inputs[:,2], kind='cubic',fill_value = 1.0, bounds_error=False)
        self.materialCurve = scipy.interpolate.interp1d(inputs[:,0], inputs[:,1], kind='cubic',fill_value = 1.0, bounds_error=False)
        self.measuredCurveInMaterial = inputs[:,0:2] #t, signal
        self.fitCounter = 0
        self.numGauss = numGauss
        self.tempPoints          = numpy.linspace(self.minT , self.T00, numOfDistribPoints)  #
        self.plotProg = plotProg

        self.xDistrib            =  self.tempPoints     


        self.do_conventional()




    def meltin_point_distrib_from_gaussians(self, params):
        return self.pore_size_distrib_from_gaussians(params)
    
    def Sb_at_temp(self, params, T = 185.0):
        """
        here T is scalar!
        deltaT = k/r
        deltaT = 2*k/d
        r in Angstrom
        """

        tempPoints = self.tempPoints
        salineCurve = self.salineCurve

        k = self.k
        mpd = self.meltin_point_distrib_from_gaussians(params) #there tempoints ...

        #d   = 2.0 * self.k /(self.T0 - tempPoints) #if needed
        #psd = mpd * 2.0*self.k/d            # this is just BTW produced
        #self.psdN = psd/ numpy.max(psd)            #is this really normaliation?


        DeltaT = T -self.T00 
        Trev =  self.T00 +(self.T00 -tempPoints + DeltaT) # = T + T0 - tempPoints


        toIntegrate = mpd * salineCurve(x = Trev) 
        s_T_        = numpy.trapz(   toIntegrate, x = tempPoints)


        """
        cumInt = scipy.integrate.cumulative_trapezoid( toIntegrate, x = tempPoints, initial = 0.0)
        toSee = numpy.zeros((len(Trev), 6 ), float  )
        toSee[:,0] = tempPoints[:]
        toSee[:,1] = mpd[:]
        toSee[:,2] = salineCurve(x = Trev)[:]
        toSee[:,3] = toIntegrate[:]
        toSee[:,4] = salineCurve(x = tempPoints)[:]
        toSee[:,5] = cumInt[:]
        """


        return s_T_
    
    
    def give_model_curve(self, params):

        measuredCurve = self.measuredCurveInMaterial 
        salineCurveInterpolator = self.salineCurve
        modelCurve = numpy.zeros(len(measuredCurve), float)

        for i  in range(0, len(measuredCurve)):
            T               = measuredCurve[i,0]
            modelSignal = self.Sb_at_temp(params, T )
            modelCurve[i] =  modelSignal

        modelCurve /= numpy.max(modelCurve)
        return modelCurve

    def residual(self, params):
        """no uncertranity given
            otherwise woudl return error/uncertranity
        """

        salineCurveInterpolator = self.salineCurve
        measuredCurve = self.measuredCurveInMaterial
        numGauss = self.numGauss

        modelCurve = self.give_model_curve(params )
    
        error = modelCurve - measuredCurve[:,1]
    
        if self.plotProg > 0:
            if (self.fitCounter % self.plotProg) == 0:
                err = numpy.linalg.norm(error)
                errtxt = "%9.7f" % err
                print("plotting...")
                itxt = "%8.8d" % self.fitCounter
                xy = numpy.zeros((len(measuredCurve),2),float)
                xy[:,0] = measuredCurve[:,0]
                xy[:,1] = modelCurve[:]
                plot_me(xyScatter =  measuredCurve  , xyLine = xy     ,title = "Melting curve: Model vs Measured. Error: " + errtxt, fileName = itxt+"meltingCurveModelVsMeasured",  xl = "T [K]", yl="normalized, CPMG-filtered signal" )
           
        if (self.fitCounter  == 0):

            err = numpy.linalg.norm(error)
            errtxt = "%9.7f" % err
            itxt = "%8.8d" % self.fitCounter
            saline = numpy.zeros((len(measuredCurve),2),float)
            saline[:,1] = self.salineCurve(measuredCurve[:,0])
            saline[:,0] = measuredCurve[:,0]
            plot_me(xyScatter = saline, xyLine = measuredCurve ,title = "Melting curve, bulk saline _in_pores_ as inputs", fileName = itxt+"meltingCurveModelVsSaline",  xl = "T [K]", yl="normalized, CPMG-filtered signal" )
            print("params: ", itxt, errtxt)
            params.pretty_print()
           
        self.fitCounter += 1
    
        return error
    
    
    
    
    
    def prepare_starting_params(self):
        """
        """
        params = Parameters()
        params.add("amplitude0", value = 0.02181, min = 0.001, max= 1)
        params.add("amplitude1", value = 0.8404, min = 0.001, max= 1)
        params.add("amplitude2", value = 0.2489, min = 0.001, max= 1)
        params.add("center0", value = 229.2, min = 185.0 , max= 250.0)
        params.add("center1", value = 272.7, min = 185.0 , max= 280.0)
        params.add("center2", value = 262.8, min = 185.0 , max= 280.0)
        params.add("sigma0", value = 8.362, min = 0.1 , max= 50.0)
        params.add("sigma1", value = 0.02301, min = 0.1 , max= 50.0)
        params.add("sigma2", value = 0.3664, min = 0.1 , max= 50.0)
        self.params = params
        print("starting params:") 
        params.pretty_print()
        return None

        params = Parameters()
        numGauss = self.numGauss
        centers    =  numpy.linspace(1,200, numGauss)
        sigmas     =  numpy.linspace(1,100, numGauss)

        largestPoreProbed = numpy.max(self.poresSizes)
        if self.guessSetting is not None:
            if len(self.guessSetting) >= numGauss: # if larger, ignore the number of gaussians

                lowDone  = False
                highDone = False
                for i in range(0, len(self.guessSetting)):
                    cn = "center"    + str(i)
                    sn = "sigma"     + str(i)
                    an = "amplitude" + str(i)
                    cnn = self.guessSetting["c"][i]
                    snn = self.guessSetting["s"][i]
                    ann = self.guessSetting["a"][i]
                    if cnn < 250.0:
                        if not lowDone:
                            params.add(cn, value= cnn   ,    min=185.0, max=250.0) 
                            params.add(sn, value= snn   ,      min=0.1,  max=50.0) 
                            params.add(an, value= ann   ,      min=0.001, max=1.0) 
                            lowDone = True
                        else:
                            params.add(cn, value= cnn   , min=0.1, max= self.lowHighDiv )
                            params.add(sn, value= snn   , min=0.1, max= 450.0)
                            params.add(an, value= ann    ,min=0.001, max=1.0)
                    elif cnn <= self.lowHighDiv:
                        params.add(cn, value= cnn   , min=0.1, max= self.lowHighDiv )
                        params.add(sn, value= snn   , min=0.1, max= 450.0)
                        params.add(an, value= ann    ,min=0.001, max=1.0)
                    elif cnn >= self.lowHighDiv:
                        if not highDone:
                            params.add(cn, value= cnn   , min=self.lowHighDiv, max= largestPoreProbed)
                            params.add(sn, value= snn   , min=1.0, max= 3000.0)
                            params.add(an, value= ann    ,min=0.001, max=1.0)
                            highDone = True
                        else:
                            params.add(cn, value= cnn   , min=0.1 * cnn, max= min(largestPoreProbed, 10.0*cnn))
                            params.add(sn, value= snn   , min=0.1 *snn , max= 10. * snn)
                            params.add(an, value= ann   , min=0.1 *ann , max= min(10. * ann, 1.0))
                           
            else:
                print("less guess params given than wanted gaussians")
                sys.exit()

        else:
            # the low gaussian
            if numGauss > 2:
                i = 0
                cn = "center"    + str(i)
                sn = "sigma"     + str(i)
                an = "amplitude" + str(i)
                params.add(cn, value= 232.1   , min=185.0, max=250.0)
                params.add(sn, value=  7.9   ,    min=0.1,  max=50.0)
                params.add(an, value=0.0192  ,    min=0.001, max=1.0)
            
            # the high (bulk) gaussian
            if numGauss > 2:
                i = 1
            else:
                i = 0
            self.lowHighDiv
            cn = "center"    + str(i)
            sn = "sigma"     + str(i)
            an = "amplitude" + str(i)
            params.add(cn, value= 275.36      , min=265, max= 275.85)
            params.add(sn, value= 0.289       ,   min=0.01, max=50.0)
            params.add(an, value=1.5528       ,   min=0.001, max=2.0)
            
            #other gaussians
            if numGauss > 2:
                contWith = 2
            else:
                contWith = 1

            for i in range(contWith, numGauss):
                cn = "center"    + str(i)
                sn = "sigma"     + str(i)
                an = "amplitude" + str(i)
                params.add(cn, value= 265.68 ,            min=250, max= 273.85)
                params.add(sn, value= 0.432  ,            min=0.01, max=50.0)
                params.add(an, value=0.2062  ,            min=0.001, max=1.0)
        self.params = params
        print("starting params:") 
        params.pretty_print()
        #ipshell()
        #sys.exit()
        return None
    





    
    def fit_pore_distr(self, algorithm ):
    
        numGauss= self.numGauss
        params = self.params

        mini = lmfit.Minimizer(self.residual, params)
        out1 = mini.minimize(method = algorithm)
        
        withResults = out1.params 
        print("\n\nResults:")
        withResults.pretty_print()
        print()

        normOfErr = numpy.linalg.norm(out1.residual) #was 0.7806
        print("norm of Err: ", normOfErr)
    

        #plot the resulting pore distribution
        #curve = self.pore_size_distrib_from_gaussians(out1.params, self.poresSizes, numGauss)
        curve = self.meltin_point_distrib_from_gaussians(out1.params) #there tempoints ...
        resultC = numpy.zeros((len(self.tempPoints),2),float)
        resultC[:,0] = self.tempPoints[:]
        resultC[:,1] = curve[:]

        highIndex = int(0.95 * len(curve))
        maxOfY = numpy.max(curve[:highIndex])

        plot_me(xyScatter =  resultC  , xyLine = resultC  ,title = "Meltin point distribution"  , fileName =
            "resulting_MP_distribution", xlim = [self.T0-40.0, self.T0 + 5.0 ], xl = r"T [K]", yl="normalized distribution", ylim = [0.0, maxOfY ])
        numpy.savetxt("resulting_MP_distributionCSV.txt", resultC)
    
    
        #now the resulting melting curve
        measuredCurve = self.measuredCurveInMaterial
        modelCurve0   = self.give_model_curve(withResults)

        xy0 = numpy.zeros((len(modelCurve0),2),float)
        xy0[:,0] = measuredCurve[:,0]
        xy0[:,1] = modelCurve0[:]
        #numpy.savetxt("xy0", xy0)
    
        errtxt = "%9.7f" % normOfErr
        plot_me(xyScatter = self.measuredCurveInMaterial, xyLine = xy0 ,title = "Melting curve: Model vs Measured. Err: " + errtxt , fileName = "meltingCurveModelVsMeasuredFinal", xl = "T [K]", yl="normalized, CPMG-filtered signal" ,  xlim = [self.T0 - 40.0, self.T0 + 5.0])
    
        #now the resulting melting point distribution
        k = self.k
        #deltaTRange = k/self.poresSizes # just for convenience, preparing T range for the sampled pore sizes


        #DeltaT = T -self.T00 
        #Trev =  self.T00 +(self.T00 -tempPoints + DeltaT)
        #DeltaT = self.T0 - self.tempPoints


        
        #ipshell()
        #sys.exit()


        self.melting_point_dist_to_pore_size_distr(mpd = curve, namePrefac = "sidi_")


    
    
if __name__ == "__main__":

    """

    guess (starting point) should look like:
    c s a #meaning center sigma amplitude, yes include the letters!!
    so:

    c          s       a   
    30.8      0.42     0.007
    18.7      2.32     0.0034
    9999.0    500.0    0.2487

    """

    import argparse

    print("got arguments:\n", sys.argv)

    parser = argparse.ArgumentParser(description='', usage='use "%(prog)s --help" for more information',  formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('inputsFN', help = """
    Text file (such as .csv), with three columns (No header expected, but can be added after #):
    #Temperature normalizedSignalInPorousSample  normalizedSignalOfSaline
    185.0       0.0014                           0.0062
    ..                                           
    ..                                           
    273.2       0.768                            0.902
    275.0       1.0                              1.0""", type = str)
    parser.add_argument('-n', '--numGauss', type = int, help = "default 3",default = 3)
    parser.add_argument('-l', '--lowHighDiv', type = float, help = "default 200.0 AA",default = 200.0)
    parser.add_argument('-k', '--kConstant', type = float, help = "Gibbs-Thomson,  default 30.0 nm K",default = 30.0)
    parser.add_argument('-i', '--numOfDistribPoints', type = int, help    = "number of point of the distribution (MP/PS), def. 400",default = 400)
    parser.add_argument('-t', '--T0forMeltingPointRef', type = float, help = "relevant for PM fitting, default 273.0",default = 273.0)
    parser.add_argument('-m', '--T0fromMaxExp',     type = float, help = "Use T0 for deltaT from supplied temperature range, default 0, say\"-m 1\" to swich on ",default = 0.0)
    parser.add_argument('-a', '--algorithm', type = str, help = """
    (Default is the global optimization > ampgo <. 
    Other global are > shgo < or > dual_annealing <. 
    For only local optimization, choose e.g. > nelder <.
    For more options see documentation to LMFIT:
    https://lmfit.github.io/lmfit-py/fitting.html)

    """
    ,default = "ampgo")

    #parser.add_argument('-g', '--guess', help = """
    #Guess in text file for the parameters of gaussian functions forming the pore-size distribution. (Here header is expected)
    #For example:
    #c          s       a
    #30.8      0.42     0.007
    #18.7      2.32     0.0034
    #9999.0    500.0    0.2487
    #
    #c s a #meaning center sigma amplitude, yes include the letters.
    #Guess is probably only good idea for a local optimization (see algorithm)
    #        """, type = str, default = None)


    parser.add_argument('-p', '--plotProgress', type = int, help = "Number of fitting steps to produce plot of fitting curve, e.g. 5000, default 0--> no intermediate plot",default = 0)
    parser.add_argument('-d', '--directPoreSizeDistribFit', type = int, help = "for direct fit of pore size distrib, default 0, set \"-d 1\" to swich on ",default = 0)


    args = parser.parse_args()

    doDirect = bool(args.directPoreSizeDistribFit)
    T0fromMaxExp = bool(args.T0fromMaxExp)
    if doDirect:
        if T0fromMaxExp:
            print("Currently not possible to use T0fromMaxExp for direct pore-size fit")
            sys.exit()
        mp = MesoPorousD( inputsFN = args.inputsFN, numGauss = args.numGauss, lowHighDiv = args.lowHighDiv, guessFN
            =args.guess, kConst10 = args.kConstant, t0 = args.T0forMeltingPointRef, numOfDistribPoints =
            args.numOfDistribPoints, maxTfromExp = T0fromMaxExp, plotProg = args.plotProgress  )
    else:
        mp = MesoPorous( inputsFN = args.inputsFN, numGauss = args.numGauss, lowHighDiv = args.lowHighDiv, guessFN
            =args.guess, kConst10 = args.kConstant, t0 = args.T0forMeltingPointRef, numOfDistribPoints =
            args.numOfDistribPoints, maxTfromExp = T0fromMaxExp, plotProg = args.plotProgress   )

    mp.prepare_starting_params()
    mp.fit_pore_distr(algorithm = args.algorithm)
    
    
    
    






